/*
    SPDX-FileCopyrightText: 2006 David Nolden <david.nolden.kde@art-master.de>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "iquickopen.h"

namespace KDevelop {
IQuickOpen::~IQuickOpen()
{
}
}
