/*
    SPDX-FileCopyrightText: 2013 Milian Wolff <mail@milianw.de>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "iprojectfilter.h"

using namespace KDevelop;

IProjectFilter::~IProjectFilter()
{

}
