/*
    SPDX-FileCopyrightText: 2025 Igor Kushnir <igorkuo@gmail.com>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "itoolviewfactoryholder.h"

KDevMI::IToolViewFactoryHolder::~IToolViewFactoryHolder() = default;
