/*
    SPDX-FileCopyrightText: 2007 Andreas Pakulat <apaku@gmx.de>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "vcsimportmetadatawidget.h"

namespace KDevelop
{

VcsImportMetadataWidget::VcsImportMetadataWidget( QWidget * parent )
    : QWidget( parent )
{
}

VcsImportMetadataWidget::~VcsImportMetadataWidget()
{
}

}

#include "moc_vcsimportmetadatawidget.cpp"
