<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="KDevAppWizard" version="4" translationDomain="kdevappwizard">
<MenuBar>
  <Menu name="project" >
    <Action name="project_new" group="project_new"/>
  </Menu>
</MenuBar>
</gui>
