/*
 * Copyright (C) 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
 * Copyright (C) 2006 Samuel Weinig <sam.weinig@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

module stylesheets {

    // Introduced in DOM Level 2:
    interface [
        CustomMarkFunction,
        GenerateIsReachable,
        CustomToJS,
        Polymorphic
    ] StyleSheet {
        readonly attribute [ConvertNullStringTo=Null] DOMString        type;
                 attribute boolean          disabled;
        readonly attribute Node             ownerNode;
        readonly attribute StyleSheet       parentStyleSheet;
        readonly attribute [ConvertNullStringTo=Null] DOMString        href;
        readonly attribute [ConvertNullStringTo=Null] DOMString        title;
        readonly attribute MediaList        media;

#if defined(LANGUAGE_CPP) && LANGUAGE_CPP
        // Extra WebCore methods exposed to allow compile-time casting in C++
        boolean isCSSStyleSheet();
#endif

    };

}
