/*
    SPDX-FileCopyrightText: 2020 Friedrich W. H. Kossebau <kossebau@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "kdevelopsessionsobserver.h"

KDevelopSessionsObserver::~KDevelopSessionsObserver() = default;
