<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kdevtestview" version="8" translationDomain="kdevtestview">
<MenuBar>
 <Menu name="run">
  <Action name="run_all_tests"
	  group="run_operations"/>
 </Menu>
</MenuBar>
</gui>
