/*
    SPDX-FileCopyrightText: 2025 Igor Kushnir <igorkuo@gmail.com>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include <multifile_shared.h>

#include <cstdio>

int main()
{
    std::printf("%d\n", squared(25));
    return 0;
}
