class foo
{
    /// "qualifiedIdentifier": "foo::bar",
    /// "isDefinition": false,
    /// "definition": {
    ///     "range": "[(11, 9), (11, 12)]",
    ///     "qualifiedIdentifier": "foo::bar"
    /// }
    int bar();
};

int foo::bar() { return 0; }
