/*
    SPDX-FileCopyrightText: 2017 Aleix Pol Gonzalez <aleixpol@kde.org>

    SPDX-License-Identifier: LGPL-2.0-only
*/

#include "iruntimecontroller.h"

KDevelop::IRuntimeController::IRuntimeController() = default;

KDevelop::IRuntimeController::~IRuntimeController() = default;

#include "moc_iruntimecontroller.cpp"
