#include <QList>

int main()
{
    QList<bool> b;
    b << false;
    QList<char> c;
    c << 50;
    QList<uchar> uc;
    uc << 50;
    QList<short> s;
    s << 50;
    QList<ushort> us;
    us << 50;
    QList<int> i;
    i << 50;
    QList<uint> ui;
    ui << 50;
    QList<long> l;
    l << 50;
    QList<ulong> ul;
    ul << 50;
    QList<qint64> i64;
    i64 << 50;
    QList<quint64> ui64;
    ui64 << 50;
    QList<float> f;
    f << 50;
    QList<double> d;
    d << 50;
    return 0;
}
