/*
 * Test license header
 * In two lines
 */
#include "testname.h"

void TestName::initTestCase()
{
    // Called before the first testfunction is executed
}

void TestName::cleanupTestCase()
{
    // Called after the last testfunction was executed
}

void TestName::init()
{
    // Called before each testfunction is executed
}

void TestName::cleanup()
{
    // Called after every testfunction
}

void TestName::firstTestCase()
{

}

void TestName::secondTestCase()
{

}

void TestName::thirdTestCase()
{

}

QTEST_MAIN(TestName)
#include "testname.moc"
#include "moc_testname.cpp"
